@@ -1466,7 +1466,7 @@ def string_histogram(iterable, min_value=None, max_value=None,
             bin_nr = bisect(bin_starts, value)-1
             try:
                 bins[bin_nr] += 1
-            except:
+            except Exception:
                 print(value, bin_nr, bin_starts)
                 raise
 
@@ -1855,10 +1855,9 @@ def generate_unique_names(prefix):
 
 
 def generate_numbered_unique_names(prefix, num=None):
-    orig_num = num
-    num = 0
-    if orig_num is None:
-        yield (num, prefix)
+    if num is None:
+        yield (0, prefix)
+        num = 0
 
     while True:
         name = "%s_%d" % (prefix, num)
@@ -1940,7 +1939,17 @@ class MinRecursionLimit(object):
         sys.setrecursionlimit(new_limit)
 
     def __exit__(self, exc_type, exc_val, exc_tb):
-        sys.setrecursionlimit(self.prev_recursion_limit)
+        # Deep recursion can produce deeply nested data structures
+        # (or long chains of to-be gc'd generators) that cannot be
+        # undergo garbage collection with a lower recursion limit.
+        #
+        # As a result, it doesn't seem possible to lower the recursion limit
+        # again after it has been raised without causing reliability issues.
+        #
+        # See https://gitlab.tiker.net/inducer/sumpy/issues/31 for
+        # context.
+
+        pass
 
 # }}}
 
@@ -1967,6 +1976,63 @@ def download_from_web_if_not_present(url, local_name=None):
 # }}}
 
 
+# {{{ find git revisions
+
+def find_git_revision(tree_root):
+    # Keep this routine self-contained so that it can be copy-pasted into
+    # setup.py.
+
+    from os.path import join, exists, abspath
+    tree_root = abspath(tree_root)
+
+    if not exists(join(tree_root, ".git")):
+        return None
+
+    # construct minimal environment
+    # stolen from
+    # https://github.com/numpy/numpy/blob/055ce3e90b50b5f9ef8cf1b8641c42e391f10735/setup.py#L70-L92
+    import os
+    env = {}
+    for k in ['SYSTEMROOT', 'PATH', 'HOME']:
+        v = os.environ.get(k)
+        if v is not None:
+            env[k] = v
+    # LANGUAGE is used on win32
+    env['LANGUAGE'] = 'C'
+    env['LANG'] = 'C'
+    env['LC_ALL'] = 'C'
+
+    from subprocess import Popen, PIPE, STDOUT
+    p = Popen(["git", "rev-parse", "HEAD"], shell=False,
+              stdin=PIPE, stdout=PIPE, stderr=STDOUT, close_fds=True,
+              cwd=tree_root, env=env)
+    (git_rev, _) = p.communicate()
+
+    import sys
+    if sys.version_info >= (3,):
+        git_rev = git_rev.decode()
+
+    git_rev = git_rev.rstrip()
+
+    retcode = p.returncode
+    assert retcode is not None
+    if retcode != 0:
+        from warnings import warn
+        warn("unable to find git revision")
+        return None
+
+    return git_rev
+
+
+def find_module_git_revision(module_file, n_levels_up):
+    from os.path import dirname, join
+    tree_root = join(*([dirname(module_file)] + [".." * n_levels_up]))
+
+    return find_git_revision(tree_root)
+
+# }}}
+
+
 def _test():
     import doctest
     doctest.testmod()
